@@ -58,6 +58,7 @@
 
  /* the command-line options */
 int block_cache, page_cache;
+int cf_enable_xattr = 0;
 float fuse_attr_timeout, fuse_entry_timeout;
 
 static void zfsfuse_getcred(fuse_req_t req, cred_t *cred)
@@ -245,6 +246,11 @@ static int int_zfs_enter(zfsvfs_t *zfsvfs) {
 
 static void zfsfuse_listxattr(fuse_req_t req, fuse_ino_t ino, size_t size)
 {
+	if (!cf_enable_xattr)
+	{
+		fuse_reply_err(req, ENOSYS);
+		return;
+	}
 	union {
 		char buf[DIRENT64_RECLEN(MAXNAMELEN)];
 		struct dirent64 dirent;
@@ -323,6 +329,11 @@ out:
 
 static void zfsfuse_setxattr(fuse_req_t req, fuse_ino_t ino, const char *name, const char *value, size_t size, int flags)
 {
+	if (!cf_enable_xattr)
+	{
+		fuse_reply_err(req, ENOSYS);
+		return;
+	}
     MY_LOOKUP_XATTR();
     // Now the idea is to create a file inside the xattr directory with the
     // wanted attribute.
@@ -372,6 +383,11 @@ out:
 static void zfsfuse_getxattr(fuse_req_t req, fuse_ino_t ino, const char *name,
 	size_t size)
 {
+	if (!cf_enable_xattr)
+	{
+		fuse_reply_err(req, ENOSYS);
+		return;
+	}
     MY_LOOKUP_XATTR();
     vnode_t *new_vp = NULL;
     error = VOP_LOOKUP(vp, (char *) name, &new_vp, NULL, 0, NULL, &cred, NULL, NULL, NULL);  
@@ -438,6 +454,11 @@ out:
 
 static void zfsfuse_removexattr(fuse_req_t req, fuse_ino_t ino, const char *name)
 {
+	if (!cf_enable_xattr)
+	{
+		fuse_reply_err(req, ENOSYS);
+		return;
+	}
     MY_LOOKUP_XATTR();
     error = VOP_REMOVE(vp, (char *) name, &cred, NULL, 0);
 
@@ -626,10 +647,12 @@ static int zfsfuse_release(fuse_req_t req, fuse_ino_t ino, struct fuse_file_info
 	int error = VOP_CLOSE(info->vp, info->flags, 1, (offset_t) 0, &cred, NULL);
     if (error)
         syslog(LOG_WARNING, "zfsfuse_release: stale inode (%s)?", strerror(error));
+	else
+	{
+		VN_RELE(info->vp);
 
-	VN_RELE(info->vp);
-
-	kmem_cache_free(file_info_cache, info);
+		kmem_cache_free(file_info_cache, info);
+	}
 
 	ZFS_EXIT(zfsvfs);
 
@@ -1857,11 +1880,8 @@ struct fuse_lowlevel_ops zfs_operations =
 	.access     = zfsfuse_access_helper,
 	.statfs     = zfsfuse_statfs,
 	.destroy    = zfsfuse_destroy,
-#if 0
-// xattr support disabled for now since it's not cached by fuse and slows things down incredibly !
 	.listxattr  = zfsfuse_listxattr,
 	.setxattr   = zfsfuse_setxattr,
 	.getxattr   = zfsfuse_getxattr,
 	.removexattr= zfsfuse_removexattr,
-#endif
 };
