@@ -31,7 +31,9 @@
 extern struct fuse_lowlevel_ops zfs_operations;
 
 /* variables documented in zfs_operations.c */
-extern int block_cache, page_cache;
+extern int block_cache;
+extern int page_cache;
+extern int cf_enable_xattr;
 extern float fuse_attr_timeout, fuse_entry_timeout;
 
 #endif
